@@ -35,9 +35,10 @@ static int filter_get_image( mlt_frame this, uint8_t **image, mlt_image_format *
 {
 	
 	mlt_filter filter = mlt_frame_pop_service( this );
+	*format = mlt_image_yuv422;
 	int error = mlt_frame_get_image( this, image, format, width, height, 1 );
 
-	if ( error == 0 && *image && *format == mlt_image_yuv422 )
+	if ( error == 0 && *image )
 	{
 		mlt_position in = mlt_filter_get_in( filter );
 		//mlt_position out = mlt_filter_get_out( filter );
